function [setOfStates, setOfStatesInMap] = makeStatesFromText(filePath, rareStatesTreshold)
% sentences - cell array

import java.util.HashMap;

setOfStates = {'*', ' '};
setOfStatesInMap = HashMap;

words = textread(filePath, '%s');
sentence = [];
j = 1;
for i = 1 : length(words)        
    word = lower(words{i});
    
    % cutting out special characters like: ,?!@#$% etc.
    word = cutOutSpecialCharacters(word);
        
    % checking if end of sentence is met (we assume: '.' can only occur at
    % the end of a word)
    if ~isempty(strfind(word, '.'))
        word = word(1 : end - 1);
    end
    
    states = extractStates(word);
    
    for j = 1 : length(states)
        state = states{j};
        % counting occurences and registering if new in setOfStatess    
        if ~(setOfStatesInMap.containsKey(state))
            setOfStatesInMap.put(state, 1);
        else
            counter = setOfStatesInMap.get(state);
            setOfStatesInMap.put(state, counter + 1);
        end    
    end
end

disp(['Words: ' num2str(length(words)) '.']);
disp(['Unique states: ' num2str(setOfStatesInMap.size()) '.']);

% taking only most frequent observations
fromMapValues = setOfStatesInMap.values().toArray();
fromMapKeys = setOfStatesInMap.keySet().toArray();

tempV = zeros(1, length(fromMapValues));
tempK = cell(length(fromMapValues));
for i = 1 : length(fromMapValues)
    tempV(i) = fromMapValues(i);
    tempK{i} = fromMapKeys(i);
end

[tempV indices] = sort(tempV, 'descend');
tempK = tempK(indices);

% cutting out rare states
index = round((1 - rareStatesTreshold) * length(tempV));
if ~isempty(index)
    tempV = tempV(1 : index(1));
    tempK = tempK(1 : index(1));
end

% putting new states into setOfStates and setOfStatesInMap
setOfStatesInMap.clear();
setOfStatesInMap.put('*', 1);
setOfStatesInMap.put(' ', 2);
for i = 1 : length(tempV)
    setOfStatesInMap.put(tempK{i}, i + 2);    
    setOfStates{i + 2} = tempK{i};
end

disp(['States built: ' num2str(length(setOfStates)) '.']);